//+------------------------------------------------------------------+
//|                                           C_Trigger_Model_01.mqh |
//|                                                      Daniel Jose |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Daniel Jose"
//+------------------------------------------------------------------+
#include "C_Manager.mqh"
//+------------------------------------------------------------------+
class C_Automaton : public C_Manager
{
	protected:
		enum eTrigger {TRIGGER_NONE, TRIGGER_BUY, TRIGGER_SELL};
	private	:
		struct st00
		{
			int		Shift,
						nBars;
			double	OverBought,
						OverSold;
		}m_Infos;
		double	m_Buff[];
		int		m_nBars,
					m_Handle;
		ENUM_TIMEFRAMES m_TF;
//+------------------------------------------------------------------+
static eTrigger m_Memory;
//+------------------------------------------------------------------+	
inline eTrigger CheckTrigger(void)
			{
				int iRet;
					
				if (((iRet = iBars(NULL, m_TF)) > m_nBars) && (m_Handle != INVALID_HANDLE))
				{
					if (CopyBuffer(m_Handle, 0, 0, m_Infos.Shift + 1, m_Buff) < m_Infos.Shift + 1) return TRIGGER_NONE;
					m_nBars = iRet;
					if ((m_Buff[0] > m_Buff[m_Infos.Shift]) && (m_Buff[0] > m_Infos.OverSold) && (m_Buff[m_Infos.Shift] < m_Infos.OverSold)) return TRIGGER_BUY;
					if ((m_Buff[0] < m_Buff[m_Infos.Shift]) && (m_Buff[0] < m_Infos.OverBought) && (m_Buff[m_Infos.Shift] > m_Infos.OverBought)) return TRIGGER_SELL;
				};
				return TRIGGER_NONE;
			}
//+------------------------------------------------------------------+	
	public	:
//+------------------------------------------------------------------+
		C_Automaton(const ulong magic, double FinanceStop, double FinanceTake, uint Leverage,
						bool IsDayTrade, double Trailing, const ENUM_TIMEFRAMES iPeriod,
						const double OverBought = 70, const double OverSold = 30, const int iShift = 1)
			:C_Manager(magic, FinanceStop, FinanceTake, Leverage, IsDayTrade, Trailing, true, 10),
			 m_TF(iPeriod),
			 m_Handle(INVALID_HANDLE)
			{
				m_Infos.Shift	= iShift;
				m_Infos.OverBought = OverBought;
				m_Infos.OverSold = OverSold;
				ArraySetAsSeries(m_Buff, true);
				m_nBars = iBars(NULL, m_TF);
				m_Handle = iRSI(NULL, m_TF, 14, PRICE_CLOSE);
			}
//+------------------------------------------------------------------+
inline virtual void Triggers(void) final
			{
				if (!CtrlTimeIsPassed()) ClosePosition(); else switch (CheckTrigger())
				{
					case TRIGGER_BUY:
						if (m_Memory == TRIGGER_SELL) ClosePosition();
						if (m_Memory != TRIGGER_BUY) ToMarket(ORDER_TYPE_BUY);
						m_Memory = TRIGGER_BUY;
						break;
					case TRIGGER_SELL:
						if (m_Memory == TRIGGER_BUY) ClosePosition();
						if (m_Memory != TRIGGER_SELL) ToMarket(ORDER_TYPE_SELL);
						m_Memory = TRIGGER_SELL;
						break;
				}
				TriggerTrailingStop();
			};
//+------------------------------------------------------------------+	
};
//+------------------------------------------------------------------+
static C_Automaton::eTrigger C_Automaton::m_Memory = TRIGGER_NONE;
//+------------------------------------------------------------------+
